/* srec.c: A reader for srecord files
 *
 * The implementation here is intended to be as small as possible,
 * so as to be suitable for use in an embedded system.  Therefore,
 * it likely does not deal well with funny input.
 *
 * (c) 1996, Dionne & Associates
 * This is free softrware, under the GPL v2
 */

#undef DEBUG

#include "srec.h"

#ifdef DEBUG
#define DPUTS(s) puts(s)
#else
#define DPUTS(s)
#endif

unsigned int 
chex(unsigned char *buf, int cnt, unsigned char *csum)
{
  unsigned int val = 0;

  while (cnt--) {

    val <<= 4;
    val += (*buf >= '0' && *buf <= '9') ?
      *buf - '0' : (*buf | 0x20) - 'a' + 0xa;

    buf++;

    if (csum && !(cnt & 1)) *csum += val & 0xff;
  }
  return val;
}

datarec(char *buf,int len, unsigned int addr, unsigned char *csum, SRBIN *sb)
{
  int i = 0;

  sb->addr = addr;
  while (len--) {
    sb->data[i++] = chex(buf, 2, csum);
    buf += 2;
  }
  if (((~(*csum)) & 0xff) != (chex(buf, 2,(void *)0) & 0xff)) return -1;

  sb->count = i;
  return 0;
}

int
srec(SRTEXT *sr, SRBIN *sb)
{
  unsigned char csum = 0;

  switch (sr->Sn) {

  case '0':
    DPUTS("A header record");
    break;

  case '1':
    DPUTS("2 byte address data record");

    if (datarec(&sr->sd.sd1.data,
		chex(&sr->count,2,&csum) - 3,
		chex(&sr->sd.sd1.addr,4,&csum),
		&csum,
		sb)) {
      DPUTS("check sum error");
      return -1;
    }

    break;

  case '2':
    DPUTS("3 byte address data record");

    if (datarec(&sr->sd.sd2.data,
		chex(&sr->count,2,&csum) - 4,
		chex(&sr->sd.sd2.addr,6,&csum),
		&csum,
		sb)) {
      DPUTS("check sum error");
      return -1;
    }

    break;

  case '3':
    DPUTS("4 byte address data record");

    if (datarec(&sr->sd.sd3.data,
		chex(&sr->count,2,&csum) - 5,
		chex(&sr->sd.sd3.addr,8,&csum),
		&csum,
		sb)) {
      DPUTS("check sum error");
      return -1;
    }

    break;

  case '5':
    DPUTS("record count");
    break;

  case '7':
    DPUTS("4 byte start address");
    break;

  case '8':
    DPUTS("3 byte start address");
    break;

  case '9':
    DPUTS("2 byte start address");
    break;

  default:
    DPUTS("unknown Srecord!");
    break;

  }
  return 0;
}

